﻿using System;
using System.Collections.Generic;
using System.ServiceModel;

using BA.WorkflowServices.Demos.Contracts;

namespace TestClient
{
    class Program
    {
        static void Main(string[] args)
        {
            testProductSupplierInCode();
            testProductSupplierWorkflowService();
            testProductDistributorWorkflowService();

            Console.WriteLine("Press any key to quit");
            Console.ReadKey();
        }

        static void testProductSupplierInCode()
        {
            Console.WriteLine("Beginning testProductSupplierInCode ...");
            Console.WriteLine();

            IProductSupplier proxy =
                ChannelFactory<IProductSupplier>.CreateChannel(
                new WSHttpBinding(),
                new EndpointAddress("http://localhost:50096/ProductSupplierInCode.svc"));

            PriceQuote quote = proxy.GetQuote(19);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(169);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(493);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(884);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(-1);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            Console.WriteLine("");

            Console.WriteLine("Completed testProductSupplierInCode.");
            Console.WriteLine();
        }

        static void testProductSupplierWorkflowService()
        {
            Console.WriteLine("Beginning testProductSupplierWorkflowService ...");
            Console.WriteLine();

            IProductSupplier proxy =
                ChannelFactory<IProductSupplier>.CreateChannel(
                new WSHttpBinding(),
                new EndpointAddress("http://localhost:50096/ProductSupplierWorkflowServices.svc"));

            PriceQuote quote = proxy.GetQuote(19);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(169);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(493);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(884);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            quote = proxy.GetQuote(-1);
            Console.WriteLine("Quote for product {0} \tprice ${1}", quote.ProductId, quote.Price);
            Console.WriteLine("");

            Console.WriteLine("Completed testProductSupplierWorkflowService.");
            Console.WriteLine();
        }

        static void testProductDistributorWorkflowService()
        {
            Console.WriteLine("Beginning testProductDistributorWorkflowService ...");
            Console.WriteLine();

            List<PriceQuote> quotes = null;

            IProductDistributor proxy1 =
                ChannelFactory<IProductDistributor>.CreateChannel(
                new WSHttpContextBinding(),
                new EndpointAddress("http://localhost:50096/ProductDistributorWorkflowServices.svc"));

            proxy1.BeginOrder();
            proxy1.AddProductToOrder(10);
            proxy1.AddProductToOrder(199);

            Console.WriteLine("Getting quotes for proxy1.");
            quotes = proxy1.GetQuotes();
            writeQuotes(quotes);
            proxy1.AddProductToOrder(976);

            IProductDistributor proxy2 =
                ChannelFactory<IProductDistributor>.CreateChannel(
                new WSHttpContextBinding(),
                new EndpointAddress("http://localhost:50096/ProductDistributorWorkflowServices.svc"));

            proxy2.BeginOrder();
            proxy2.AddProductToOrder(456);
            proxy2.AddProductToOrder(14);

            Console.WriteLine("Getting quotes for proxy2.");
            quotes = proxy2.GetQuotes();
            writeQuotes(quotes);
            proxy2.AddProductToOrder(678);

            Console.WriteLine("Finishing quotes for proxy1.");
            quotes = proxy1.PlaceOrder();
            writeQuotes(quotes);

            Console.WriteLine("Finishing quotes for proxy2.");
            quotes = proxy2.PlaceOrder();
            writeQuotes(quotes);

            Console.WriteLine("Completed testProductDistributorWorkflowService.");
            Console.WriteLine();
        }

        static void writeQuotes(List<PriceQuote> quotes)
        {
            foreach (PriceQuote quote in quotes)
            {
                Console.WriteLine("\tProduct {0} \tPrice {1}", quote.ProductId, quote.Price);
            }
            Console.WriteLine();
        }
    }
}
